@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
 #
 # Podcastparser: A simple, fast and efficient podcast parser
-# Copyright (c) 2012, 2013, 2014, 2018, Thomas Perl <m@thp.io>
+# Copyright (c) 2012, 2013, 2014, 2018, 2020 Thomas Perl <m@thp.io>
+# Copyright (c) 2016, 2017, 2018, 2019, 2020 Eric Le Lay <elelay@macports.org>
+# Copyright (c) 2020 E.S. Rosenberg <es.rosenberg+openu@gmail.com>
 #
 # Permission to use, copy, modify, and/or distribute this software for any
 # purpose with or without fee is hereby granted, provided that the above
@@ -20,7 +22,7 @@
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Thomas Perl <m@thp.io>'
-__version__ = '0.6.4'
+__version__ = '0.6.6'
 __website__ = 'http://gpodder.org/podcastparser/'
 __license__ = 'ISC License'
 
@@ -91,7 +93,8 @@ class RSS(Target):
 class PodcastItem(Target):
     def end(self, handler, text):
         by_published = lambda entry: entry.get('published')
-        handler.data['episodes'].sort(key=by_published, reverse=True)
+        order = 'type' not in handler.data or handler.data['type'] != 'serial'
+        handler.data['episodes'].sort(key=by_published, reverse=order)
         if handler.max_episodes:
             episodes = handler.data['episodes'][:handler.max_episodes]
             handler.data['episodes'] = episodes
@@ -104,6 +107,15 @@ class PodcastAttr(Target):
         handler.set_podcast_attr(self.key, self.filter_func(text))
 
 
+class PodcastAttrType(Target):
+    WANT_TEXT = True
+
+    def end(self, handler, text):
+        value = self.filter_func(text)
+        if value in ('episodic', 'serial'):
+            handler.set_podcast_attr(self.key, value)
+
+
 class PodcastAttrRelativeLink(PodcastAttr):
     def end(self, handler, text):
         text = urlparse.urljoin(handler.base, text)
@@ -235,20 +247,42 @@ class AtomContent(Target):
             handler.set_episode_attr('description', squash_whitespace(text))
 
 
+class RSSItemDescription(Target):
+    """
+    RSS 2.0 almost encourages to put html content in item/description
+    but content:encoded is the better source of html content and itunes:summary
+    is known to contain the short textual description of the item.
+    So use a heuristic to attribute text to either description or description_html,
+    without overriding existing values.
+    """
+    WANT_TEXT = True
+
+    def __init__(self):
+        self._want_content = False
+
+    def end(self, handler, text):
+        if is_html(text):
+            if not handler.get_episode_attr('description_html'):
+                handler.set_episode_attr('description_html', text.strip())
+        elif not handler.get_episode_attr('description'):
+            # don't overwrite itunes:summary?
+            handler.set_episode_attr('description', squash_whitespace(text))
+
+
 class PodloveChapters(Target):
     SUPPORTED_VERSIONS = ('1.1', '1.2')
 
     def start(self, handler, attrs):
         version = attrs.get('version', '1.1')
         if version not in PodloveChapters.SUPPORTED_VERSIONS:
-            logger.warn('Possible incompatible chapters version: %s', version)
+            logger.warning('Possible incompatible chapters version: %s', version)
 
 
 class PodloveChapter(Target):
     def start(self, handler, attrs):
         # Both the start and title attributes are mandatory
         if attrs.get('start') is None or attrs.get('title') is None:
-            logger.warn('Invalid chapter (missing start and/or and title)')
+            logger.warning('Invalid chapter (missing start and/or and title)')
             return
 
         chapter = {
@@ -361,7 +395,7 @@ class Namespace():
             namespace_uri = self.lookup(namespace)
             if namespace_uri is None:
                 # Use of "itunes:duration" without xmlns:itunes="..."
-                logger.warn('No namespace defined for "%s:%s"', namespace,
+                logger.warning('No namespace defined for "%s:%s"', namespace,
                             name)
                 return '%s:%s' % (namespace, name)
 
@@ -369,7 +403,7 @@ class Namespace():
             prefix = self.NAMESPACES.get(namespace_uri)
             if prefix is None and namespace:
                 # Proper use of namespaces, but unknown namespace
-                # logger.warn('Unknown namespace: %s', namespace_uri)
+                # logger.warning('Unknown namespace: %s', namespace_uri)
                 # We prefix the tag name here to make sure that it does not
                 # match any other tag below if we can't recognize the namespace
                 name = '!%s:%s' % (namespace, name)
@@ -466,7 +500,7 @@ def parse_time(value):
         try:
             seconds = int(value)
         except ValueError:
-            logger.warn('Could not parse time value: "%s"', value)
+            logger.warning('Could not parse time value: "%s"', value)
             return 0
 
     return (int(hours) * 60 + int(minutes)) * 60 + int(seconds)
@@ -554,9 +588,9 @@ def parse_pubdate(text):
             pubtimeseconds = int(mktime_tz(parsed))
             return pubtimeseconds
         except(OverflowError,ValueError):
-            logger.warn('bad pubdate %s is before epoch or after end of time (2038)',parsed)
+            logger.warning('bad pubdate %s is before epoch or after end of time (2038)',parsed)
             return 0
-        
+
     try:
         parsed = time.strptime(text[:19], '%Y-%m-%dT%H:%M:%S')
         if parsed is not None:
@@ -590,13 +624,14 @@ MAPPING = {
     'rss/channel/description': PodcastAttr('description', squash_whitespace),
     'rss/channel/image/url': PodcastAttrRelativeLink('cover_url'),
     'rss/channel/itunes:image': PodcastAttrFromHref('cover_url'),
+    'rss/channel/itunes:type': PodcastAttrType('type', squash_whitespace),
     'rss/channel/atom:link': PodcastAtomLink(),
 
     'rss/channel/item': EpisodeItem(),
     'rss/channel/item/guid': EpisodeGuid('guid'),
     'rss/channel/item/title': EpisodeAttr('title', squash_whitespace),
     'rss/channel/item/link': EpisodeAttrRelativeLink('link'),
-    'rss/channel/item/description': EpisodeAttr('description', squash_whitespace),
+    'rss/channel/item/description': RSSItemDescription(),
     'rss/channel/item/itunes:summary': EpisodeAttr('description', squash_whitespace),
     'rss/channel/item/media:description': EpisodeAttr('description', squash_whitespace),
     'rss/channel/item/itunes:subtitle': EpisodeAttr('subtitle', squash_whitespace),
@@ -604,6 +639,7 @@ MAPPING = {
     'rss/channel/item/itunes:duration': EpisodeAttr('total_time', parse_time),
     'rss/channel/item/pubDate': EpisodeAttr('published', parse_pubdate),
     'rss/channel/item/atom:link': AtomLink(),
+    'rss/channel/item/itunes:image': EpisodeAttrFromHref('episode_art_url'),
 
     'rss/channel/item/media:content': Enclosure('fileSize'),
     'rss/channel/item/enclosure': Enclosure('length'),
@@ -614,7 +650,7 @@ MAPPING = {
     'atom:feed': PodcastItem(),
     'atom:feed/atom:title': PodcastAttr('title', squash_whitespace),
     'atom:feed/atom:subtitle': PodcastAttr('description', squash_whitespace),
-    'atom:feed/atom:icon': PodcastAttr('cover_url'),
+    'atom:feed/atom:icon': PodcastAttrRelativeLink('cover_url'),
     'atom:feed/atom:link': PodcastAtomLink(),
     'atom:feed/atom:entry': EpisodeItem(),
     'atom:feed/atom:entry/atom:id': EpisodeAttr('guid'),
@@ -636,7 +672,7 @@ VALID_ROOTS = set(path.split('/')[0] for path in MAPPING.keys())
 class FeedParseError(sax.SAXParseException, ValueError):
     """
     Exception raised when asked to parse an invalid feed
-    
+
     This exception allows users of this library to catch exceptions
     without having to import the XML parsing library themselves.
     """
@@ -691,12 +727,12 @@ class PodcastHandler(sax.handler.ContentHandler):
             del entry['chapters']
 
         # Ensures `description` does not contain HTML
-        if 'description' in entry and is_html(entry['description']):
+        if is_html(entry['description']):
             if 'description_html' not in entry:
                 entry['description_html'] = entry['description']
             entry['description'] = ''
 
-        # Sets `description` to stripped `description_html` when absent
+        # Sets `description` to stripped `description_html` when empty
         if 'description_html' in entry and not entry['description']:
             entry['description'] = remove_html_tags(entry['description_html'])
 
@@ -853,12 +889,17 @@ def normalize_feed_url(url):
     # urlunsplit might return "a slighty different, but equivalent URL"
     return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
+HTML_TEST = re.compile('<[a-z][a-z0-9]*(?:\s.*?>|\/?>)', re.IGNORECASE | re.DOTALL)
 def is_html(text):
+    """Heuristically tell if text is HTML
+
+    By looking for an open tag (more or less:)
+    >>> is_html('<h1>HELLO</h1>')
+    True
+    >>> is_html('a < b < c')
+    False
     """
-    Tests whether the given string contains HTML encoded data
-    """
-    html_test = re.compile(r'<[a-z][\s\S]*>', re.IGNORECASE)
-    return bool(html_test.search(text))
+    return bool(HTML_TEST.search(text))
 
 def remove_html_tags(html):
     """
